#    Copyright © 2021 Andrei Puchko
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.


# from q2gui import q2form

import os
import time
from PyQt6.QtWidgets import (
    QApplication,
    QWidget,
    QMainWindow,
    QToolButton,
    QToolBar,
    QFileDialog,
    QTabWidget,
    QSplitter,
    QMdiArea,
    QMdiSubWindow,
    QHBoxLayout,
)

from PyQt6.QtCore import QEvent, Qt, QCoreApplication, QTimer, QRectF, QPoint, QEventLoop, QByteArray
from PyQt6.QtTest import QTest
from PyQt6.QtGui import (
    QFontMetrics,
    QIcon,
    QFont,
    QBrush,
    QColor,
    QShortcut,
    QKeySequence,
    QAction,
    QPixmap,
    QPainter,
)
from PyQt6.QtSvg import QSvgRenderer

from q2gui.pyqt6.q2window import Q2QtWindow
from q2gui.pyqt6.q2style import Q2Style
from q2gui.pyqt6.widgets.q2frame import q2frame
from q2gui.pyqt6.widgets.q2text import q2text
from q2gui.pyqt6.widgets.q2button import q2button
from q2gui.q2utils import int_


import q2gui.q2app as q2app
from q2gui.q2icons import icons


class stdout_widget(q2frame):
    def __init__(self, mode="h"):
        super().__init__({"column": "/h", "label": "Output"})
        self.stdout_widget = q2text(self.make_meta({}))

        self.toolbar_frame = q2frame({"column": "/v"})
        self.closeButton = q2button(self.make_meta(column="hide", label="❌", mess="Hide output"))
        self.closeButton.clicked.connect(lambda: self.hide())

        self.cleanButton = q2button(self.make_meta(label="🧹", mess="Clean output"))
        self.cleanButton.clicked.connect(lambda: [self.stdout_widget.clear(), self.stdout_widget.set_focus()])

        self.toolbar_frame.insert_widget(widget=self.closeButton)
        self.toolbar_frame.insert_widget(widget=self.cleanButton)

        self.insert_widget(widget=self.stdout_widget)
        self.insert_widget(widget=self.toolbar_frame)
        self.setVisible(False)
        self.setStyleSheet("margin: 0.5em 2px 2px 2px; padding:1px")

    def write(self, text):
        if not self.isVisible():
            self.setVisible(True)
        from PyQt6.QtGui import QTextCursor

        self.stdout_widget.moveCursor(QTextCursor.MoveOperation.End)
        q2app.q2_app.process_events()
        self.stdout_widget.insertPlainText(text)


class Q2Toolbars:
    def __init__(self, q2_app):
        self.toolbars = {}
        self.q2_app: Q2App = q2_app
        self.visible = True

    def clear(self):
        for toolbar in self.toolbars:
            x, y = self.toolbars[toolbar].pos().x(), self.toolbars[toolbar].pos().y()
            floating = self.toolbars[toolbar].isFloating()
            area = self.q2_app.toolBarArea(self.toolbars[toolbar]).value
            # orientation = self.toolbars[toolbar].orientation().value
            self.q2_app.settings.set(f"toolbar-{toolbar}", "x", f"{x}")
            self.q2_app.settings.set(f"toolbar-{toolbar}", "y", f"{y}")
            self.q2_app.settings.set(f"toolbar-{toolbar}", "floating", f"{floating}")
            self.q2_app.settings.set(f"toolbar-{toolbar}", "area", f"{area}")
            # self.q2_app.settings.set(f"toolbar-{toolbar}", "orientation", f"{orientation}")
            self.toolbars[toolbar].clear()
            self.q2_app.removeToolBar(self.toolbars[toolbar])
        self.toolbars = {}

    def move(self, pos, old_pos):
        delta_x = pos.x() - old_pos.x()
        delta_y = pos.y() - old_pos.y()

        for toolbar in self.toolbars:
            if self.toolbars[toolbar].isFloating():
                pos_x = self.toolbars[toolbar].pos().x()
                pos_y = self.toolbars[toolbar].pos().y()
                self.toolbars[toolbar].move(pos_x + delta_x, pos_y + delta_y)

    def addAction(self, action, toolbar=""):
        if toolbar not in self.toolbars:
            self.toolbars[toolbar] = QToolBar()
            x = int_(self.q2_app.settings.get(f"toolbar-{toolbar}", "x", None))
            y = int_(self.q2_app.settings.get(f"toolbar-{toolbar}", "y", None))
            floating = self.q2_app.settings.get(f"toolbar-{toolbar}", "floating", None)
            area = self.q2_app.settings.get(f"toolbar-{toolbar}", "area", None)
            # orientation = self.q2_app.settings.get(f"toolbar-{toolbar}", "orientation", None)
            if area:
                if area == "ToolBarArea.TopToolBarArea":
                    area = Qt.ToolBarArea.TopToolBarArea
                elif area == "ToolBarArea.BottomToolBarAre":
                    area = Qt.ToolBarArea.BottomToolBarArea
                elif area == "ToolBarArea.LeftToolBarArea":
                    area = Qt.ToolBarArea.LeftToolBarArea
                elif area == "ToolBarArea.RightToolBarArea":
                    area = Qt.ToolBarArea.RightToolBarArea
                elif area.isdigit():
                    area = int(area)
                    if area == Qt.ToolBarArea.TopToolBarArea.value:
                        area = Qt.ToolBarArea.TopToolBarArea
                    elif area == Qt.ToolBarArea.BottomToolBarArea.value:
                        area = Qt.ToolBarArea.BottomToolBarArea
                    elif area == Qt.ToolBarArea.LeftToolBarArea.value:
                        area = Qt.ToolBarArea.LeftToolBarArea
                    elif area == Qt.ToolBarArea.RightToolBarArea.value:
                        area = Qt.ToolBarArea.RightToolBarArea
                    else:
                        area = Qt.ToolBarArea.TopToolBarArea
            if area is not None:
                self.q2_app.addToolBar(area, self.toolbars[toolbar])
            else:
                self.q2_app.addToolBar(self.toolbars[toolbar])
            if floating == "True":
                self.toolbars[toolbar].setWindowFlags(
                    Qt.WindowType.Tool
                    | Qt.WindowType.FramelessWindowHint
                    | Qt.WindowType.X11BypassWindowManagerHint
                )
                self.toolbars[toolbar].move(int(x), int(y))
                self.toolbars[toolbar].setOrientation(Qt.Orientation.Horizontal)
        if isinstance(action, QAction):
            self.toolbars[toolbar].addAction(action)
        elif isinstance(action, QToolButton):
            self.toolbars[toolbar].addWidget(action)

    def isVisible(self):
        return self.visible

    def isEnabled(self):
        toolbars_flag = [self.toolbars[x].isEnabled() for x in self.toolbars]
        if toolbars_flag:
            return max(toolbars_flag)
        else:
            return False

    def hide(self):
        for x in self.toolbars:
            self.toolbars[x].hide()
        self.visible = False

    def show(self):
        for x in self.toolbars:
            self.toolbars[x].show()
        self.visible = True

    def setDisabled(self, mode):
        for x in self.toolbars:
            self.toolbars[x].setDisabled(mode)


class Q2App(QMainWindow, q2app.Q2App, Q2QtWindow):
    class Q2TabWidget(QTabWidget):
        def __init__(self, parent):
            super().__init__(parent)
            self.main_window: Q2App = parent
            self.addTab(QWidget(self), "+")
            self.setAttribute(Qt.WidgetAttribute.WA_DeleteOnClose)
            self.setObjectName("main_tab_widget")
            self.prev_index = None
            self.tab_focus_widget = {}
            self.tabBar().setObjectName("main_tab_bar")

            self.corner_widget = QWidget(self)
            self.corner_widget.setLayout(QHBoxLayout())

            self.closeButton = QToolButton(self)
            self.closeButton.setIcon(self.main_window.get_engine_icon("close"))
            self.closeButton.setObjectName("tab_bar_close_button")
            self.closeButton.setContentsMargins(0, 0, 0, 0)
            self.closeButton.clicked.connect(self.closeSubWindow)

            self.restore_button = QToolButton(self)
            self.restore_button.setIcon(self.main_window.get_engine_icon("restore"))
            self.restore_button.setContentsMargins(0, 0, 0, 0)
            self.restore_button.clicked.connect(self.show_mdi_normal)
            self.show_mdi_normal_button(False)

            self.corner_widget.layout().setContentsMargins(0, 0, 0, 0)
            self.corner_widget.layout().setAlignment(Qt.AlignmentFlag.AlignRight)
            self.corner_widget.layout().addWidget(self.restore_button)
            self.corner_widget.layout().addWidget(self.closeButton)
            self.setCornerWidget(self.corner_widget)

            self.currentChanged.connect(self.restore_tab_focus_widget)

        def hide(self) -> None:
            self.tabBar().setDisabled(True)
            self.tabBar().setTabVisible(self.tabBar().count() - 1, False)
            # self.tabBar().hide()
            # self.closeButton.hide()
            # return super().hide()

        def show(self) -> None:
            self.tabBar().setEnabled(True)
            self.tabBar().setTabVisible(self.tabBar().count() - 1, True)
            self.setUpdatesEnabled(False)
            self.tabBar().hide()
            self.tabBar().show()
            self.setUpdatesEnabled(True)
            # self.tabBar().show()
            # self.closeButton.show()
            # return super().show()

        def save_tab_focus_widget(self, widget):
            self.tab_focus_widget[self.currentIndex()] = widget

        def restore_tab_focus_widget(self):
            if self.currentIndex() == self.count() - 1:
                self.addTab()
                return
            show_mdi_normal_button = False
            if self.currentWidget().activeSubWindow():
                if self.currentWidget().activeSubWindow().isMaximized():
                    show_mdi_normal_button = True
            self.show_mdi_normal_button(show_mdi_normal_button)
            focus_widget = self.tab_focus_widget.get(self.currentIndex(), self)
            if focus_widget:
                try:
                    focus_widget.setFocus()
                except Exception:
                    pass

        def closeSubWindow(self):
            # print(">", len(set(self.main_window.QApplication.allWidgets())))
            currentTabIndex = self.currentIndex()
            if wlist := self.currentWidget().subWindowList():
                wlist[-1].close()
            elif self.count() > 2:  # close tab if them >2
                self.setCurrentIndex(currentTabIndex - 1)
                # self.removeTab(currentTabIndex)
                self.widget(currentTabIndex).deleteLater()
            # print(">>", len(set(self.main_window.QApplication.allWidgets())))

        def show_mdi_normal(self):
            self.currentWidget().activeSubWindow().showNormal()
            self.currentWidget().activeSubWindow().setWindowFlag(Qt.WindowType.FramelessWindowHint, False)

        def show_mdi_normal_button(self, mode=False):
            self.restore_button.setVisible(mode)

        def get_subwindow_count(self):
            return sum([len(self.widget(x).subWindowList()) for x in range(self.count() - 1)])

        def addTab(self, widget=None, label=" = "):
            if not widget:
                widget = QMdiArea(self)
                self.set_tab_background(widget)
                widget.form_level = 0
                widget.setOption(QMdiArea.AreaOption.DontMaximizeSubWindowOnActivation)
                widget.setHorizontalScrollBarPolicy(Qt.ScrollBarPolicy.ScrollBarAsNeeded)
                widget.setVerticalScrollBarPolicy(Qt.ScrollBarPolicy.ScrollBarAsNeeded)
                widget.setAttribute(Qt.WidgetAttribute.WA_DeleteOnClose, True)

            self.insertTab(self.count() - 1, widget, label)
            self.setCurrentIndex(self.count() - 2)
            self.currentWidget().setAttribute(Qt.WidgetAttribute.WA_DeleteOnClose, True)
            if self.count() > 1:
                self.main_window.on_new_tab()

        def set_tab_background(self, widget):
            if isinstance(widget, QMdiArea):
                if self.main_window.q2style.color_mode == "clean":
                    widget.setBackground(QBrush(QApplication.palette().dark()))
                else:
                    widget.setBackground(
                        QBrush(QColor(self.main_window.q2style.get_style("background_disabled")))
                    )

        def set_tabs_backround(self):
            for x in range(self.count()):
                self.set_tab_background(self.widget(x))

    def __init__(self, title=""):
        if QCoreApplication.startingUp():  # one and only QApplication allowed
            self.QApplication = QApplication([])
            self.QApplication.setAttribute(Qt.ApplicationAttribute.AA_DontUseNativeMenuBar)
        QMainWindow.__init__(self)
        Q2QtWindow.__init__(self)
        self.q2_tabwidget = self.Q2TabWidget(self)
        # self.q2_toolbar = QToolBar(self)
        self.q2_toolbar = Q2Toolbars(self)
        self.stdout_widget = stdout_widget()

        self.Q2Style = Q2Style
        q2app.Q2App.__init__(self)

        if not hasattr(QApplication, "_mw_count"):
            QApplication._mw_count = 0
            QApplication._mw_list = []
        QApplication._mw_count += 1
        QApplication._mw_list.append(self)
        self.closing = False

        self.central_widget = QSplitter(Qt.Orientation.Vertical)
        self.setCentralWidget(self.central_widget)
        self.centralWidget().setObjectName("central_widget")

        # self.addToolBar(self.q2_toolbar)

        self.central_widget.addWidget(self.q2_tabwidget)
        self.central_widget.addWidget(self.stdout_widget)
        self.central_widget.setSizes([10])

        self.statusBar().setVisible(True)
        self.set_title(title)

        QApplication.instance().focusChanged.connect(self.focus_changed)
        QApplication.instance().focusChanged.connect(self.save_tab_focus_widget)

        self.next_tab_shortcut = QShortcut(QKeySequence("Ctrl+Tab"), self)
        self.prev_tab_shortcut = QShortcut(QKeySequence("Ctrl+Shift+Tab"), self)
        self.next_tab_shortcut.activated.connect(
            lambda: (
                self.q2_tabwidget.setCurrentIndex(self.q2_tabwidget.currentIndex() + 1)
                if self.q2_tabwidget.tabBar().isVisible()
                else None
            )
        )
        self.prev_tab_shortcut.activated.connect(
            lambda: (
                self.q2_tabwidget.setCurrentIndex(self.q2_tabwidget.currentIndex() - 1)
                if self.q2_tabwidget.tabBar().isVisible()
                else None
            )
        )

        # replace static methods for instance
        self.get_open_file_dialoq = self._get_open_file_dialoq
        self.get_save_file_dialoq = self._get_save_file_dialoq
        self._last_get_file_path = None

    def set_clipboard(self, text):
        QApplication.clipboard().setText(text)

    def get_stdout_height(self):
        return self.stdout_widget.height()

    def set_font(self, font_name="", font_size=12):
        QApplication.setFont(QFont(font_name, font_size))

    def save_tab_focus_widget(self):
        self.q2_tabwidget.save_tab_focus_widget(self.focus_widget())

    def get_self(self):
        if QApplication.activeWindow():
            return QApplication.activeWindow()
        else:
            return self

    def subwindow_count_changed(self):
        return self.q2_tabwidget.get_subwindow_count()

    def disable_menu(self, menu_path=""):
        action = self._main_menu.get(menu_path)
        if action:
            action.setDisabled(True)

    def enable_menu(self, menu_path=""):
        action = self._main_menu.get(menu_path)
        if action:
            action.setEnabled(True)

    # def eventFilter(self, obj, ev: QEvent):
    #     if ev.type() == QEvent.Type.Close:
    #         if not obj.isEnabled():
    #             ev.ignore()
    #             return True
    #     return super().eventFilter(obj, ev)

    def moveEvent(self, ev):
        self.q2_toolbar.move(ev.pos(), ev.oldPos())
        return super().moveEvent(ev)

    def show_form(self, form=None, modal="modal"):
        # form.heap = q2app.Q2Heap()
        form.heap.modal = modal
        form.heap.prev_mdi_window = self.q2_tabwidget.currentWidget().activeSubWindow()
        form.heap.prev_focus_widget = QApplication.focusWidget()
        form.heap.prev_tabbar_text = self.get_tabbar_text()
        prev_q2_form = self.get_current_q2_form()
        if form.q2_form.prev_form is None and prev_q2_form:
            form.q2_form.prev_form = prev_q2_form

        form_mdi_subwindow: QMdiSubWindow = self.q2_tabwidget.currentWidget().addSubWindow(form)
        form_mdi_subwindow.setWindowIcon(self.windowIcon())
        tmp_icon = QPixmap(1, 1)
        tmp_icon.fill(Qt.GlobalColor.transparent)
        form_mdi_subwindow.setWindowIcon(QIcon(tmp_icon))

        # form.installEventFilter(self)
        self.subwindow_count_changed()

        if modal != "" and form.heap.prev_mdi_window:  # mdiarea normal window
            form.heap.prev_mdi_window.setDisabled(True)

        self.set_tabbar_text(form.window_title)

        if modal == "":  # mdiarea normal window
            form.show()
        else:
            if modal == "super":  # real modal dialog
                form.heap.prev_toolbar_enabled = self.is_toolbar_enabled()
                form.heap.prev_menubar_enabled = self.is_menubar_enabled()
                form.heap.prev_tabbar_enabled = self.is_tabbar_enabled()
                self.disable_toolbar(True)
                self.disable_menubar(True)
                self.disable_tabbar(True)
            # form.show()
            # while form.not_closed:
            #     time.sleep(0.005)
            #     self.process_events()

            loop = QEventLoop()
            form.destroyed.connect(loop.quit)
            form.show()
            if loop:
                loop.exec()  # blocks until form is destroyed

            if modal == "super":  # real modal dialog
                self.enable_toolbar(form.heap.prev_toolbar_enabled)
                self.enable_menubar(form.heap.prev_menubar_enabled)
                self.enable_tabbar(form.heap.prev_tabbar_enabled)
        self.subwindow_count_changed()

    def dpi(self):
        return self.physicalDpiX()

    def get_current_q2_form(self):
        if self.q2_tabwidget.currentWidget().activeSubWindow() is None:
            return None
        active_widget = self.q2_tabwidget.currentWidget().activeSubWindow().widget()
        if hasattr(active_widget, "q2_form"):
            return active_widget.q2_form

    def disable_current_form(self, mode=True):
        if self.q2_tabwidget.currentWidget().subWindowList():
            prev_mdi_window = self.q2_tabwidget.currentWidget().subWindowList()[-1]
            if prev_mdi_window:
                prev_mdi_window.setDisabled(mode)
                prev_mdi_window.setFocus()

    def build_menu(self):
        # self.menu_list = super().reorder_menu(self.menu_list)
        super().build_menu()
        self._main_menu = {}
        QMainWindow.menuBar(self).clear()
        self.q2_toolbar.clear()
        QMainWindow.menuBar(self).show()
        for x in self.menu_list:
            _path = x["TEXT"]
            if _path == "" or _path in self._main_menu:
                continue
            prevNode = "|".join(_path.split("|")[:-1])
            topic = _path.split("|")[-1]
            if _path.count("|") == 0:  # first in chain - menu bar
                node = QMainWindow.menuBar(self)
            else:
                node = self._main_menu.get(prevNode)
                if node is None:
                    node = QMainWindow.menuBar(self)
            if _path.endswith("-"):
                node.addSeparator()
            elif x["WORKER"]:
                self._main_menu[_path] = node.addAction(topic)
                self._main_menu[_path].setMenuRole(QAction.MenuRole.NoRole)
                self._main_menu[_path].triggered.connect(x["WORKER"])

                self._main_menu[_path].setIcon(self.get_engine_icon(x["ICON"]))

                if x["TOOLBAR"]:
                    self.q2_toolbar.addAction(self._main_menu[_path], _path.split("|")[0])
            else:
                self._main_menu[_path] = node.addMenu(topic)
        # Show as context menu
        self.setContextMenuPolicy(Qt.ContextMenuPolicy.ActionsContextMenu)
        for a in self.actions():
            self.removeAction(a)
        self.addActions(self.menuBar().actions())

    def get_engine_icon(self, icon_text):
        if not icon_text:
            return QIcon()

        tmp_icon = self.get_icon(q2app.GRID_ACTION_ICON)
        if tmp_icon:
            icon_size = QIcon(tmp_icon).availableSizes()[0].width()
        else:
            icon_size = 24

        if (_icon_key := icon_text.split(".")[0]) in icons:
            return self.make_svg_icon(icons[_icon_key], icon_size)
        elif _icon := self.get_icon(icon_text):
            return QIcon(self.get_icon(_icon))
        else:
            return self.make_text_icon(icon_text[0], icon_size)

    def make_text_icon(self, icon_text, icon_size=24):
        pix = QPixmap(icon_size, icon_size)
        pix.fill(Qt.GlobalColor.transparent)
        font = QFont("Arial")
        font.setWeight(QFont.Weight.Bold)
        font.setPixelSize(icon_size)
        painter = QPainter(pix)
        painter.setFont(font)
        painter.drawText(
            QRectF(0, 0, icon_size, icon_size),
            Qt.AlignmentFlag.AlignCenter | Qt.AlignmentFlag.AlignHCenter,
            icon_text[:1],
        )
        painter.end()
        return QIcon(pix)

    def make_svg_icon(self, svg_text, size=24):
        # if mode == "dark":
        #     c = "#ffffff"
        # else:
        #     color = "#000000"
        # if mode == "dark":
        #     svg_text = svg_text.replace('stroke="black"', 'stroke="red"')
        renderer = QSvgRenderer(QByteArray(svg_text.encode("utf-8")))
        pixmap = QPixmap(size, size)
        pixmap.fill(Qt.GlobalColor.transparent)
        painter = QPainter(pixmap)
        target_rect = QRectF(0, 1, size, size)
        renderer.render(painter, target_rect)
        painter.end()
        return QIcon(pixmap)

    def set_color_mode(self, color_mode=None):
        q2app.Q2App.set_color_mode(self, color_mode)
        self.q2_tabwidget.set_tabs_backround()

    def focus_widget(self):
        return QApplication.focusWidget()

    def get_clipboard_text(self):
        return QApplication.clipboard().text()

    def set_clipboard_text(self, text):
        return QApplication.clipboard().setText(text)

    def set_style_sheet(self, style=None):
        if os.path.isfile(self.style_file):
            try:
                with open(self.style_file, "r") as style_data:
                    local_style = style_data.read()
            except Exception:
                local_style = ""
        else:
            local_style = ""
        self.setStyleSheet(style + local_style)

    def add_style_sheet(self, style):
        current_style = self.styleSheet() + f"{style}"
        self.setStyleSheet(current_style)

    def lock(self):
        self.menuBar().setDisabled(True)
        self.q2_toolbar.setDisabled(True)
        self.q2_tabwidget.setDisabled(True)

    def unlock(self):
        self.menuBar().setDisabled(False)
        self.q2_toolbar.setDisabled(False)
        self.q2_tabwidget.setDisabled(False)

    def set_icon(self, icon_path):
        self.icon = icon_path
        # self.setWindowIcon(QIcon(self.icon))
        self.setWindowIcon(self.get_engine_icon("q2gui"))

    def process_events(self):
        QApplication.processEvents()

    def show_menubar(self, mode=True):
        q2app.Q2App.show_menubar(self)
        if mode:
            QMainWindow.menuBar(self).show()
        else:
            QMainWindow.menuBar(self).hide()

    def is_menubar_visible(self):
        return QMainWindow.menuBar(self).isVisible()

    def is_menubar_enabled(self):
        return QMainWindow.menuBar(self).isEnabled()

    def show_toolbar(self, mode=True):
        q2app.Q2App.show_toolbar(self)
        if mode:
            self.q2_toolbar.show()
        else:
            self.q2_toolbar.hide()

    def disable_toolbar(self, mode=True):
        self.q2_toolbar.setDisabled(True if mode else False)

    def disable_menubar(self, mode=True):
        QMainWindow.menuBar(self).setDisabled(True if mode else False)

    def disable_tabbar(self, mode=True):
        self.q2_tabwidget.tabBar().setDisabled(True if mode else False)

    def is_toolbar_visible(self):
        return self.q2_toolbar.isVisible()

    def is_toolbar_enabled(self):
        return self.q2_toolbar.isEnabled()

    def show_tabbar(self, mode=True):
        q2app.Q2App.show_tabbar(self)
        if mode:
            self.q2_tabwidget.show()
        else:
            self.q2_tabwidget.hide()

    def is_tabbar_visible(self):
        return self.q2_tabwidget.tabBar().isEnabled()

    def is_tabbar_enabled(self):
        return self.q2_tabwidget.tabBar().isEnabled()

    def get_tabbar_text(self):
        return self.q2_tabwidget.tabBar().tabText(self.q2_tabwidget.currentIndex())

    def show_statusbar_mess(self, text=""):
        self.statusBar().showMessage(f"{text}")

    def clear_statusbar(self):
        self.statusBar().clearMessage()

    def get_statusbar_mess(self):
        return self.statusBar().currentMessage()

    def set_tabbar_text(self, text=""):
        # text = text.strip().split("\n")[0][:50]
        self.q2_tabwidget.tabBar().setTabText(self.q2_tabwidget.currentIndex(), text)
        self.setUpdatesEnabled(False)
        self.q2_tabwidget.tabBar().hide()
        self.q2_tabwidget.tabBar().show()
        self.setUpdatesEnabled(True)

    def show_statusbar(self, mode=True):
        q2app.Q2App.show_statusbar(self)
        if mode:
            self.statusBar().show()
        else:
            self.statusBar().hide()

    def is_statusbar_visible(self):
        return self.statusBar().isVisible()

    def get_char_width(self, char="w"):
        return QFontMetrics(self.font()).horizontalAdvance(char)

    def get_char_height(self):
        return QFontMetrics(self.font()).height()

    @staticmethod
    def get_open_file_dialoq(header=q2app.DIALOG_OPEN_FILE_TITLE, path="", filter=""):
        if path == "":
            path = os.path.expanduser("~/Desktop")
        rez = QFileDialog.getOpenFileName(None, header, path, filter)
        return rez

    def _get_open_file_dialoq(self, header=q2app.DIALOG_OPEN_FILE_TITLE, path="", filter=""):
        if self._last_get_file_path and not path:
            path = self._last_get_file_path
        rez = Q2App.get_open_file_dialoq(header, path, filter)
        if rez:
            self._last_get_file_path = os.path.dirname(rez[0])
        QApplication.setActiveWindow(self)
        return rez

    @staticmethod
    def get_save_file_dialoq(header=q2app.DIALOG_SAVE_FILE_TITLE, path="", filter="", confirm_overwrite=True):
        if path == "":
            path = os.path.expanduser("~/Desktop")
        if confirm_overwrite:
            rez = QFileDialog.getSaveFileName(None, header, path, filter)
        else:
            rez = QFileDialog.getSaveFileName(
                None, header, path, filter, options=QFileDialog.Option.DontConfirmOverwrite
            )
        return rez

    def _get_save_file_dialoq(
        self, header=q2app.DIALOG_SAVE_FILE_TITLE, path="", filter="", confirm_overwrite=True
    ):
        if self._last_get_file_path and not path:
            path = self._last_get_file_path
        rez = Q2App.get_save_file_dialoq(header, path, filter, confirm_overwrite)
        if rez:
            self._last_get_file_path = os.path.dirname(rez[0])
        QApplication.setActiveWindow(self)
        return rez

    def _wait_for_show(self):
        while QApplication.activeWindow() is None:
            QTimer.singleShot(100, self._wait_for_show)
            return
        self.process_events()
        self.add_new_tab()
        self.process_events()
        self.on_start()

    def keyboard_modifiers(self):
        self.process_events()
        modifiers = QApplication.keyboardModifiers()
        rez = []
        if modifiers == Qt.KeyboardModifier.ShiftModifier:
            rez.append("shift")
        elif modifiers == Qt.KeyboardModifier.ControlModifier:
            rez.append("control")
        elif modifiers == (Qt.KeyboardModifier.AltModifier):
            rez.append("alt")
        return "+".join(rez)

    def save_geometry(self, settings):
        Q2QtWindow.save_geometry(self, settings)

    def run(self):
        # self.restore_geometry(self.settings)
        Q2QtWindow.restore_geometry(self, self.settings)
        self.show()
        super().run()
        self._wait_for_show()
        if len(QApplication.allWindows()) == 1:
            QApplication.instance().exec()

    def click_menu(self, *path):
        menu_paths = []
        menu_tree = {}

        def build_menu_tree():
            """Builds a tree representation of the menu system."""

            def build_menu_items(menu, path):
                items = {}
                for action in menu.actions():
                    if action.isSeparator():
                        continue
                    if action.menu():  # Nested menu
                        _path = path[:]
                        _path.append(action.text())
                        items[action.text()] = {
                            "menu": action.menu(),
                            "items": build_menu_items(action.menu(), _path),
                        }
                    else:  # Regular action
                        items[action.text()] = {}
                        menu_paths.append(path + [action.text()])
                return items

            for menu_action in self.menuBar().actions():
                menu = menu_action.menu()
                if menu:
                    menu_tree[menu_action.text()] = {
                        "menu": menu,
                        "items": build_menu_items(menu, [menu_action.text()]),
                    }

        def click_menubar():
            for menubar_action in self.menuBar().actions():
                if menubar_action.text() == path[0]:
                    break
            menu_rect = self.menuBar().actionGeometry(menubar_action)
            menu_center = QPoint(menu_rect.x() + 2, menu_rect.y() + 2)
            QTest.mouseClick(self.menuBar(), Qt.MouseButton.LeftButton, pos=menu_center)
            QApplication.processEvents()
            menu = menu_tree[path[0]]
            return menu

        def click_submenu(menu, submenu_text):
            if menu is None:
                return menu
            for action in menu["menu"].actions():
                if action.text() == submenu_text:
                    menu_rect = menu["menu"].actionGeometry(action)
                    menu_center = QPoint(
                        menu_rect.x() + menu_rect.width() // 2, menu_rect.y() + menu_rect.height() // 2
                    )
                    QTest.mouseClick(menu["menu"], Qt.MouseButton.LeftButton, pos=menu_center)
                    QApplication.processEvents()
                    if menu["items"][submenu_text]:
                        QTest.qWait(100)
                    return menu["items"][submenu_text]

        build_menu_tree()
        if list(path) in menu_paths:
            menu = click_menubar()
            for submenu in path[1:]:
                if menu:
                    menu = click_submenu(menu, submenu)
        QApplication.processEvents()

    def add_new_tab(self):
        self.q2_tabwidget.addTab()

    def show(self):
        QMainWindow.show(self)

    def on_new_tab(self):
        return super().on_new_tab()

    def showEvent(self, event):
        event.accept()
        super().showEvent(event)

    def closeEvent(self, event: QEvent):
        if not self.closing:
            self.close()
        event.accept()

    def close(self):
        self.closing = True
        self.q2_toolbar.clear()
        self.save_geometry(self.settings)
        super().close()
        QApplication._mw_count -= 1
        QMainWindow.close(self)
        if QApplication._mw_count <= 0:
            os._exit(0)



#    Copyright © 2021 Andrei Puchko
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.


from PyQt6.QtWidgets import (
    QTableWidget,
    QFrame,
    QTableView,
    QStyledItemDelegate,
    QAbstractItemView,
    QSizePolicy,
    QVBoxLayout,
)
from PyQt6.QtGui import QKeyEvent, QPalette, QPainter, QFontMetrics, QColor

from PyQt6.QtCore import (
    Qt,
    QAbstractTableModel,
    QVariant,
    QItemSelectionModel,
    QSortFilterProxyModel,
)

from q2gui.pyqt6.q2window import q2_align
from q2gui.q2utils import int_
from q2gui.q2model import Q2Model
from q2gui.pyqt6.widgets.q2lookup import q2lookup
from q2gui.pyqt6.widgets.q2button import q2button
from q2gui.pyqt6.widgets.q2line import q2line
from q2gui.pyqt6.widgets.q2check import q2check
from q2gui.pyqt6.widgets.q2frame import q2frame


sort_ascend_char = "▲"
sort_decend_char = "▼"
filtered_char = "☑"


class q2Delegate(QStyledItemDelegate):
    def paint(self, painter: QPainter, option, index):
        if self.parent().currentIndex().column() == index.column():
            color = option.palette.color(QPalette.ColorRole.AlternateBase).darker(900)
            color.setAlpha(int(color.alpha() / 5))
            painter.fillRect(option.rect, color)
        super().paint(painter, option, index)


class q2grid(QTableView):
    class Q2TableModel(QAbstractTableModel):
        def __init__(self, q2_model):
            super().__init__(parent=None)
            self.q2_model: Q2Model = q2_model
            self._q2_model_refresh = self.q2_model.refresh
            self.q2_model.refresh = self.refresh

        def rowCount(self, parent=None):
            return self.q2_model.row_count()

        def columnCount(self, parent=None):
            return self.q2_model.column_count()

        def refresh(self):
            self.beginResetModel()
            self.endResetModel()
            self._q2_model_refresh()
            self.apply_col_filter()

        def data(self, index, role=Qt.ItemDataRole.DisplayRole):
            control = self.q2_model.meta[index.column()].get("control")
            if role == Qt.ItemDataRole.DisplayRole:
                if control == "check":
                    return QVariant()
                else:
                    return QVariant(self.q2_model.data(index.row(), index.column()))
            elif role == Qt.ItemDataRole.TextAlignmentRole:
                return QVariant(
                    Qt.AlignmentFlag.AlignVCenter | q2_align[str(self.q2_model.alignment(index.column()))]
                )
            elif (
                role == Qt.ItemDataRole.BackgroundRole
                and self.q2_model.q2_bcolor
                and (color := int_(self.q2_model.get_record(index.row()).get("q2_bcolor", 0))) != 0
            ):
                if index.row() in self.q2_model.q2_bcolors:
                    _color = self.q2_model.q2_bcolors[index.row()]
                else:
                    self.q2_model.q2_bcolors[index.row()] = _color = f"#{color:06x}"
                return QColor(_color)
            elif (
                role == Qt.ItemDataRole.ForegroundRole
                and self.q2_model.q2_fcolor
                and (color := int_(self.q2_model.get_record(index.row()).get("q2_fcolor", 0))) != 0
            ):
                if index.row() in self.q2_model.q2_fcolors:
                    _color = self.q2_model.q2_fcolors[index.row()]
                else:
                    self.q2_model.q2_fcolors[index.row()] = _color = f"#{color:06x}"
                return QColor(_color)
            elif role == Qt.ItemDataRole.CheckStateRole:
                if control == "check":
                    if self.q2_model.data(index.row(), index.column()):
                        return Qt.CheckState.Checked
                    else:
                        return Qt.CheckState.Unchecked
            else:
                return QVariant()

        def headerData(self, col, orientation, role=Qt.ItemDataRole.DisplayRole):
            if orientation == Qt.Orientation.Horizontal and role == Qt.ItemDataRole.DisplayRole:
                sort_char = ""
                if self.q2_model.columns[col] in self.q2_model.order_text:
                    if "desc" in self.q2_model.order_text:
                        sort_char = sort_decend_char
                    else:
                        sort_char = sort_ascend_char
                if col in self.q2_model.filtered_columns:
                    sort_char += f" {filtered_char} "
                return sort_char + self.q2_model.headers[col]
            # elif orientation == Qt.Orientation.Horizontal and role == Qt.ItemDataRole.BackgroundRole:
            #     return QBrush(Qt.GlobalColor.red)
            elif orientation == Qt.Orientation.Vertical and role == Qt.ItemDataRole.DisplayRole:
                if col in self.q2_model.hidden_rows and not self.q2grid.isRowHidden(col):
                    self.q2grid.setRowHidden(col, True)
                return QVariant("")
            else:
                return QVariant()

        def flags(self, index):
            control = self.q2_model.meta[index.column()].get("control")
            flags = super().flags(index)
            if control == "check":
                flags |= Qt.ItemFlag.ItemIsUserCheckable
            return flags

        def apply_col_filter(self, filter_values=None, column=None):
            for row in range(self.rowCount()):
                self.q2grid.setRowHidden(row, False)
            self.q2_model.hidden_rows = []
            if filter_values is not None:
                self.q2_model.columns_filter_values[column] = filter_values
                if self.q2_model.columns_filter_values[column] == []:
                    del self.q2_model.columns_filter_values[column]
                self.q2_model.filtered_columns = list(self.q2_model.columns_filter_values.keys())
            for row in range(self.rowCount()):
                if row in self.q2_model.hidden_rows:
                    continue
                for col, values in self.q2_model.columns_filter_values.items():
                    if self.q2_model.data(row, col) not in values:
                        self.q2_model.hidden_rows.append(row)
                        # self.q2grid.setRowHidden(row, True)
                        break

    def __init__(self, meta):
        super().__init__()
        self.meta = meta

        self.q2_form = self.meta.get("form")
        self.q2_model = self.q2_form.model

        if self.q2_model.column_count() > 1:
            self.setItemDelegate(q2Delegate(self))
        self.setTabKeyNavigation(False)
        self.setSizePolicy(QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Expanding)

        self.horizontalHeader().setSectionsMovable(True)
        self.horizontalHeader().setDefaultAlignment(q2_align["7"])
        self.horizontalHeader().sectionClicked.connect(self.q2_form.grid_header_clicked)

        self.setAlternatingRowColors(True)
        self.setSelectionBehavior(QAbstractItemView.SelectionBehavior.SelectRows)
        self.setSelectionMode(QAbstractItemView.SelectionMode.ExtendedSelection)
        self.setContextMenuPolicy(Qt.ContextMenuPolicy.ActionsContextMenu)
        self.doubleClicked.connect(self.q2_form.grid_double_clicked)
        self.setModel(self.Q2TableModel(self.q2_form.model))
        self.model().q2grid = self
        self.horizontalHeader().setContextMenuPolicy(Qt.ContextMenuPolicy.CustomContextMenu)
        self.horizontalHeader().customContextMenuRequested.connect(self.activate_header_menu)

    def activate_header_menu(self, pos):
        self.set_index(self.currentIndex().row(), self.horizontalHeader().logicalIndexAt(pos))
        self.meta["q2_app"].process_events()
        if self.row_count() > 10000:
            return
        column_manager = q2_col_manager(self)
        column_manager.show()

    def showEvent(self, event):
        h = self.fontMetrics().height()
        self.verticalHeader().setDefaultSectionSize(h)
        self.verticalHeader().setMinimumSectionSize(h + h // 4)
        return super().showEvent(event)

    def apply_col_filter(self, filter_values):
        self.model().apply_col_filter(filter_values, self.currentIndex().column())
        self.set_index(0)

    def currentChanged(self, current, previous):
        super().currentChanged(current, previous)
        self.model().dataChanged.emit(current, previous)
        self.model().dataChanged.emit(previous, current)
        self.q2_form._grid_index_changed(self.currentIndex().row(), self.currentIndex().column())

    def current_index(self):
        return self.currentIndex().row(), self.currentIndex().column()

    def set_focus(self):
        self.setFocus()

    def has_focus(self):
        return self.hasFocus()

    def is_enabled(self):
        return self.isEnabled()

    def row_count(self):
        return self.model().rowCount()

    def column_count(self):
        return self.model().columnCount()

    def set_index(self, row, column=None):
        _last_row = self.currentIndex().row()
        self.clearSelection()
        if row < 0:
            row = 0
        elif row > self.row_count() - 1:
            row = self.row_count() - 1

        if column is None:
            column = self.currentIndex().column()
        elif column < 0:
            column = 0
        elif column > self.column_count() - 1:
            column = self.column_count() - 1

        if self.model().q2_model.filtered_columns:
            if row > _last_row:
                while self.isRowHidden(row) and row < self.model().rowCount() - 1:
                    row += 1
            elif row < _last_row and row > 0:
                while self.isRowHidden(row) and row > 0:
                    row -= 1

            if row == 0:
                while self.isRowHidden(row):
                    row += 1
            elif row == self.model().rowCount() - 1:
                while self.isRowHidden(row):
                    row -= 1

        self.setCurrentIndex(self.model().index(row, column))

    def keyPressEvent(self, event):
        event.accept()
        # if ev.key() in [Qt.Key.Key_F] and ev.modifiers() == Qt.ControlModifier:
        #     self.searchText()
        # if event.key() in [Qt.Key.Key_Asterisk]:
        if (
            event.text()
            and event.key() not in (Qt.Key.Key_Escape, Qt.Key.Key_Enter, Qt.Key.Key_Return, Qt.Key.Key_Space)
            and self.model().rowCount() >= 1
            and event.modifiers() != Qt.KeyboardModifier.ControlModifier
            and event.modifiers() != Qt.KeyboardModifier.AltModifier
        ):
            lookup_widget = q2_grid_lookup(self, event.text(), meta=self.meta)
            lookup_widget.show(self, self.currentIndex().column())
        else:
            super().keyPressEvent(event)

    def get_selected_rows(self):
        return [x.row() for x in self.selectionModel().selectedRows() if not self.isRowHidden(x.row())]

    def set_selected_rows(self, index_list):
        self.clearSelection()
        indexes = [self.model().index(row, 0) for row in index_list if not self.isRowHidden(row)]
        mode = QItemSelectionModel.SelectionFlag.Select | QItemSelectionModel.SelectionFlag.Rows
        [self.selectionModel().select(index, mode) for index in indexes]

    def get_columns_headers(self):
        rez = {}
        hohe = self.horizontalHeader()
        for x in range(0, hohe.count()):
            # col_header = hohe.model().headerData(x, Qt.Orientation.Horizontal, Qt.ItemDataRole.DisplayRole)
            col_header = self.q2_model.headers[x]
            rez[col_header] = x
        return rez

    def get_columns_settings(self):
        rez = []
        hohe = self.horizontalHeader()
        for x in range(0, hohe.count()):
            # header = hohe.model().headerData(x, Qt.Orientation.Horizontal, Qt.ItemDataRole.DisplayRole)
            header = self.q2_model.headers[x]
            width = self.columnWidth(x)
            pos = hohe.visualIndex(x)
            rez.append({"name": header, "data": f"{pos}, {width}"})
        return rez

    def set_column_settings(self, col_settings):
        headers = self.get_columns_headers()
        for x, col_size in col_settings.items():
            if "," not in col_settings[x]:
                continue
            column_pos, column_width = [int_(sz) for sz in col_size.split(",")]
            self.setColumnWidth(headers.get(x), column_width)
            old_visual = self.horizontalHeader().visualIndex(int_(headers[x]))
            self.horizontalHeader().moveSection(old_visual, column_pos)
        self.set_index(0, self.horizontalHeader().logicalIndex(0))


class q2_grid_lookup(q2lookup):
    def lookup_list_selected(self):
        self.q2_grid.set_index(self.found_rows[self.lookup_list.currentRow()][0])
        self.close()

    def lookup_search(self):
        self.lookup_list.clear()
        self.found_rows = self.q2_model.lookup(self.q2_model_column, self.lookup_edit.get_text())
        for x in self.found_rows:
            self.lookup_list.addItem(f"{x[1]}")

    def show(self, q2_grid, column):
        self.q2_grid = q2_grid
        self.q2_model_column = column
        self.q2_model = q2_grid.q2_model
        super().show()
        self.lookup_edit.setCursorPosition(1)

    def set_geometry(self):
        parent = self.parent()
        rect = parent.visualRect(parent.currentIndex())
        rect.moveTop(parent.horizontalHeader().height() + 2)
        rect.moveLeft(parent.verticalHeader().width() + rect.x() + 2)
        pos = rect.topLeft()
        pos = parent.mapToGlobal(pos)
        self.setFixedWidth(parent.width() - rect.x())
        self.move(pos)


class q2_col_manager(QFrame):

    class content_view(QTableView):
        class content_view_model(QAbstractTableModel):
            def __init__(self, parent):
                super().__init__(parent)
                # self._content = set()
                q2grid = self.parent().parent().q2grid
                column = self.parent().parent()._column
                self._content = q2grid.model().q2_model.unique_column_values(column)

            def rowCount(self, parent=None):
                return len(self._content)

            def columnCount(self, parent=None):
                return 1

            def data(self, index, role=Qt.ItemDataRole.DisplayRole):
                if role == Qt.ItemDataRole.DisplayRole:
                    return self._content[index.row()]["v"]
                elif role == Qt.ItemDataRole.TextAlignmentRole:
                    return Qt.AlignmentFlag.AlignLeft | Qt.AlignmentFlag.AlignVCenter
                elif role == Qt.ItemDataRole.CheckStateRole:
                    if self._content[index.row()].get("c", True):
                        return Qt.CheckState.Checked
                    else:
                        return Qt.CheckState.Unchecked

            def toogle_check(self, index):
                self._content[index.row()]["c"] = not self._content[index.row()].get("c", True)
                self.dataChanged.emit(index, index)

        def __init__(self, parent):
            super().__init__(parent)
            self.setShowGrid(False)
            self.setTabKeyNavigation(False)
            self.setSelectionMode(QTableWidget.SelectionMode.SingleSelection)
            self._model = q2_col_manager.content_view.content_view_model(self)
            self.proxy = QSortFilterProxyModel()
            self.proxy.setSourceModel(self._model)
            self.proxy.setFilterCaseSensitivity(Qt.CaseSensitivity.CaseInsensitive)
            self.setModel(self.proxy)

            self.horizontalHeader().hide()
            self.verticalHeader().hide()

            self.setHorizontalScrollMode(QTableView.ScrollMode.ScrollPerPixel)
            self.verticalHeader().setDefaultSectionSize(QFontMetrics(self.font()).height())
            self.resizeColumnsToContents()
            self.clicked.connect(self.on_click)

        def on_search_changed(self, text):
            self.proxy.setFilterWildcard(text)
            # self.proxy.setFilterRegularExpression(text)

        def get_checked(self):
            rez = []
            for x in range(self.model().rowCount()):
                row = self.proxy.mapToSource(self.proxy.index(x, 0)).row()
                if self._model._content[row].get("c", True):
                    rez.append(self._model._content[row]["v"])
            return rez

        def check_all(self, state):
            for x in range(self.model().rowCount()):
                self._model._content[self.proxy.mapToSource(self.proxy.index(x, 0)).row()]["c"] = state
            self._model.beginResetModel()
            self._model.endResetModel()

        def invert(self, state):
            for x in range(self.model().rowCount()):
                self._model._content[self.proxy.mapToSource(self.proxy.index(x, 0)).row()]["c"] = (
                    not self._model._content[self.proxy.mapToSource(self.proxy.index(x, 0)).row()]["c"]
                )
            self._model.beginResetModel()
            self._model.endResetModel()

        def keyPressEvent(self, event: QKeyEvent):
            if event.key() in (Qt.Key.Key_Enter, Qt.Key.Key_Return, Qt.Key.Key_Space):
                self.on_click(self.currentIndex())
                self.setCurrentIndex(self.model().index(self.currentIndex().row() + 1, 0))
            return super().keyPressEvent(event)

        def on_click(self, index):
            self._model.toogle_check(self.proxy.mapToSource(index))

    def __init__(self, parent):
        parent.setDisabled(True)
        super().__init__(parent, Qt.WindowType.Popup)
        self.setWindowModality(Qt.WindowModality.WindowModal)
        self.filter_values = None
        self.q2grid = parent
        self._column = parent.currentIndex().column()
        self.setLayout(QVBoxLayout())
        self.layout().setContentsMargins(0, 0, 0, 0)
        self.layout().setSpacing(0)
        self.setAttribute(Qt.WidgetAttribute.WA_DeleteOnClose)
        self.setObjectName("col_manager")
        self.setStyleSheet("QFrame#col_manager {border: 1px solid palette(Mid);border-radius: 0.25ex;}")
        self.meta = {}
        # self.q2grid.meta["q2_app"].process_events()

        self.frame_order = q2frame({"column": "/v", "label": "Order"})

        self.order_az = q2button(self.meta)
        self.order_az.set_minimum_width(20)
        self.order_az.set_text("Sort A - Z")
        self.order_az.clicked.connect(self._order_az)

        self.order_za = q2button(self.meta)
        self.order_za.set_text("Sort Z - A")
        self.order_za.clicked.connect(self._order_za)

        self.frame_order.layout().addWidget(self.order_az)
        self.frame_order.layout().addWidget(self.order_za)

        self.filter_by_cond = q2button(self.meta)
        self.filter_by_cond.set_text("Filter by condition")

        self.filter_by_values = q2button(self.meta)
        self.filter_by_values.set_text("Filter by values")
        self.filter_by_values.set_minimum_width(20)

        self._search_item = q2line()
        self._search_item.setPlaceholderText("Search items...")
        self._search_item.textChanged.connect(self.on_search_changed)

        self.content_viewer = q2_col_manager.content_view(self)

        self.frame_filter = q2frame({"column": "/v", "label": "Filter"})
        self.check_all = q2check({"label": "Check all", "data": True})
        self.check_all.toggled.connect(self.content_viewer.check_all)
        self.invert = q2check({"label": "Invert", "data": True})
        self.invert.toggled.connect(self.content_viewer.invert)

        self.frame_filter_buttons = q2frame({"column": "/h", "label": "-"})

        self.filter_ok_button = q2button(self.meta)
        self.filter_ok_button.set_text("Ok")
        self.filter_ok_button.setObjectName("_ok_button")
        self.filter_ok_button.clicked.connect(self.apply_filter)

        self.filter_cancel_button = q2button(self.meta)
        self.filter_cancel_button.set_text("Cancel")
        self.filter_cancel_button.setObjectName("_cancel_button")
        self.filter_cancel_button.clicked.connect(self.close)
        self.frame_filter_buttons.layout().addWidget(self.filter_ok_button)
        self.frame_filter_buttons.layout().addWidget(self.filter_cancel_button)

        self.frame_filter.layout().addWidget(self._search_item)
        self.frame_toogler = q2frame({"column": "/h", "label": "-"})
        self.frame_toogler.layout().addWidget(self.check_all)
        self.frame_toogler.layout().addWidget(self.invert)
        self.frame_filter.layout().addWidget(self.frame_toogler)
        self.frame_filter.layout().addWidget(self.content_viewer)
        self.frame_filter.layout().addWidget(self.frame_filter_buttons)

        self.layout().addWidget(self.frame_order)
        self.layout().addWidget(self.frame_filter)

    def on_search_changed(self, text):
        self.content_viewer.on_search_changed(text)

    def apply_filter(self):
        checked = self.content_viewer.get_checked()
        if len(checked) == self.content_viewer._model.rowCount():
            checked = []
        self.q2grid.apply_col_filter(checked)
        self.close()

    def check_all_changed(self):
        self.content_viewer.check_all(self.check_all.isChecked())

    def closeEvent(self, a0):
        self.q2grid.setEnabled(True)
        self.q2grid.set_focus()
        return super().closeEvent(a0)

    def _order_az(self):
        self.hide()
        self.q2grid.q2_form.grid_header_clicked(self._column, "AZ")
        self.close()

    def _order_za(self):
        self.hide()
        self.q2grid.q2_form.grid_header_clicked(self._column, "ZA")
        self.close()

    def show(self) -> None:
        super().show()
        self.set_geometry()
        self._search_item.setFocus()

    def set_geometry(self):
        rect = self.q2grid.visualRect(self.q2grid.currentIndex())
        rect.moveTop(self.q2grid.horizontalHeader().height() + 2)
        rect.moveLeft(self.q2grid.verticalHeader().width() + rect.x() + 2)
        if self.content_viewer.columnWidth(0) > self.q2grid.width():
            w = self.content_viewer.columnWidth(0)
            if w > self.q2grid.width():
                w = self.q2grid.width() - rect.topLeft().x()
            self.setFixedWidth(w)
        pos = rect.topLeft()
        if pos.x() + self.width() > self.q2grid.rect().width():
            pos.setX(self.q2grid.rect().width() - self.width())
        pos = self.q2grid.mapToGlobal(pos)
        self.move(pos)

    def keyPressEvent(self, event: QKeyEvent):
        if event.key() == Qt.Key.Key_PageDown:
            self.apply_filter()
        return super().keyPressEvent(event)



#    Copyright © 2021 Andrei Puchko
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

import sys


from PyQt6.QtWidgets import (
    QVBoxLayout,
    QWidget,
)
from PyQt6.QtGui import QKeyEvent

from PyQt6.QtCore import Qt, QTimer

from q2gui.pyqt6.widgets.q2line import q2line
from q2gui.pyqt6.widgets.q2list import q2list


class q2lookup(QWidget):
    def __init__(self, parent, text, meta={}):
        super().__init__(parent, Qt.WindowType.Popup)
        self.setLayout(QVBoxLayout())
        self.layout().setContentsMargins(0, 0, 0, 0)
        self.setAttribute(Qt.WidgetAttribute.WA_DeleteOnClose)
        self.meta = {"q2_app": meta.get("q2_app")}
        self.lookup_edit = q2line(self.meta)
        self.lookup_list = q2list(self.meta)
        self.layout().addWidget(self.lookup_edit)
        self.layout().addWidget(self.lookup_list)
        self.lookup_edit.set_text("" if text == "*" else text)
        self.lookup_edit.set_focus()

        self.timer = QTimer()
        self.timer.setSingleShot(True)
        self.timer.setInterval(500)
        self.timer.timeout.connect(self.lookup_search)

        self.lookup_edit.textChanged.connect(self.lookup_text_changed)
        self.lookup_edit.returnPressed.connect(self.lookup_edit_return_pressed)

        self.lookup_list.itemActivated.connect(self.lookup_list_selected)

        self.set_geometry()

    # def lookup_list_selected(self):
    #     print("Method lookup_list_selected has to be implemented...")

    def set_geometry(self):
        print("Method set_geometry has to be implemented...")

    # def show(self, column):
    #     self.q2_model_column = column
    #     return super().show()

    def lookup_list_selected(self):
        # print(self.lookup_list.currentItem().text())
        self.close()

    def lookup_search(self):
        self.lookup_list.clear()
        for x in range(6):
            self.lookup_list.addItem(f"{x} Method lookup_search has to be implemented...")

    def lookup_edit_return_pressed(self):
        self.timer.stop()
        self.timer.timeout.emit()
        self.lookup_list.setFocus()

    def lookup_text_changed(self):
        if len(self.lookup_edit.get_text()) > 1:
            self.timer.start()

    def keyPressEvent(self, event: QKeyEvent):
        if event.key() == Qt.Key.Key_Down and self.lookup_edit.hasFocus():
            event.accept()
            self.lookup_list.setCurrentRow(0)
            self.lookup_list.setFocus()
        elif (
            event.key() == Qt.Key.Key_Up
            and self.lookup_list.hasFocus()
            and self.lookup_list.currentRow() == 0
        ):
            self.lookup_edit.setFocus()
            event.accept()
        else:
            return super().keyPressEvent(event)



#    Copyright © 2021 Andrei Puchko
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.


from PyQt6.QtWidgets import QTimeEdit, QAbstractSpinBox
from PyQt6.QtCore import QTime, Qt
from q2gui.pyqt6.q2widget import Q2Widget

# from q2gui.q2utils import int_


class q2time(QTimeEdit, Q2Widget):
    def __init__(self, meta):
        super().__init__(meta)
        self.meta = meta
        self.set_text(meta.get("data"))
        if self.meta.get("valid"):
            self.timeChanged.connect(self.meta.get("valid"))

        self.set_fixed_height()
        self.setButtonSymbols(QAbstractSpinBox.ButtonSymbols.PlusMinus)

    def set_text(self, text):
        if text.count(":") == 2:
            self.setTime(QTime.fromString(text, "HH:mm:ss"))
        elif text.count(":") == 1:
            self.setTime(QTime.fromString(text, "HH:mm"))
        else:
            self.setTime(QTime.fromString(text, "HH"))

    def get_text(self):
        return self.text()

    def keyPressEvent(self, ev):
        if ev.key() in [
            Qt.Key.Key_Down,
            Qt.Key.Key_Up,
            Qt.Key.Key_PageDown,
            Qt.Key.Key_PageUp,
        ]:
            ev.ignore()
        elif ev.key() == Qt.Key.Key_Plus:
            tm: QTime = self.time()
            if self.currentSection() == self.Section.HourSection:
                tm = tm.addSecs(3600)
                self.setTime(tm)
            if self.currentSection() == self.Section.MinuteSection:
                tm = tm.addSecs(60)
                self.setTime(tm)
        elif ev.key() == Qt.Key.Key_Minus:
            tm: QTime = self.time()
            if self.currentSection() == self.Section.HourSection:
                tm = tm.addSecs(-3600)
                self.setTime(tm)
            if self.currentSection() == self.Section.MinuteSection:
                tm = tm.addSecs(-60)
                self.setTime(tm)
        else:
            super().keyPressEvent(ev)



#    Copyright © 2021 Andrei Puchko
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.


from PyQt6.QtWidgets import QTabBar, QTabWidget
from PyQt6.QtGui import QKeySequence, QShortcut
from PyQt6.QtCore import Qt

from q2gui.pyqt6.q2window import Q2Frame
from q2gui.pyqt6.q2widget import Q2Widget
from q2gui.q2utils import int_


class Q2TabBar(QTabBar, Q2Widget):
    def get_text(self):
        return self.tabText(self.currentIndex())


class q2tab(QTabWidget, Q2Widget, Q2Frame):
    def __init__(self, meta):
        super().__init__(meta)
        self.setTabBar(Q2TabBar())
        self.meta = meta

        self.set_tabbar_position(meta.get("alignment", 7))

        self.next_tab_hotkey = QShortcut(QKeySequence("Ctrl+PgDown"), self)
        self.next_tab_hotkey.activated.connect(self.next_tab)

        self.prev_tab_hotkey = QShortcut(QKeySequence("Ctrl+PgUp"), self)
        self.prev_tab_hotkey.activated.connect(self.prev_tab)

    def set_tabbar_position(self, pos):
        pos = str(pos)
        qss = ""
        if "4" in pos:  # left
            self.setTabPosition(QTabWidget.TabPosition.West)
            if "7" in pos:
                qss = "left"
            elif "1" in pos:
                qss = "right"
            else:
                qss = "center"
        elif "2" in pos:  # bottom
            self.setTabPosition(QTabWidget.TabPosition.South)
            if "1" in pos:
                qss = "left"
            elif "3" in pos:
                qss = "right"
            else:
                qss = "center"
        elif "6" in pos:  # right
            self.setTabPosition(QTabWidget.TabPosition.East)
            if "9" in pos:
                qss = "left"
            elif "3" in pos:
                qss = "right"
            else:
                qss = "center"
        else:
            if "9" in pos:
                qss = "right"
            elif "8" in pos:
                qss = "center"
        if qss:
            self.setStyleSheet("QTabWidget::tab-bar { alignment: %s;}" % qss)

    def next_tab(self):
        self.setCurrentIndex(self.currentIndex() + 1)

    def prev_tab(self):
        self.setCurrentIndex(self.currentIndex() - 1)

    def set_enabled(self, mode=True):
        self.tabBar().setEnabled(mode)

    def set_disabled(self, mode=True):
        self.tabBar().setDisabled(mode)

    def set_tab(self, index=0):
        if isinstance(index, str):
            for x in range(self.tabBar().count()):
                if self.tabBar().tabText(x) == index:
                    index = x
                    break
        else:
            index = int_(index)
            if index >= self.count():
                index = self.count() - 1
            elif index < 0:
                index = 0
        self.setCurrentIndex(index)

    def minimumSizeHint(self):
        self.setMinimumHeight(super().minimumSizeHint().height())
        return super().minimumSizeHint()

    def set_shortcuts_local(self):
        self.next_tab_hotkey.setContext(Qt.ShortcutContext.WidgetWithChildrenShortcut)
        self.prev_tab_hotkey.setContext(Qt.ShortcutContext.WidgetWithChildrenShortcut)

    def add_tab(self, widget, text=""):
        self.addTab(widget, text)

    def get_text(self):
        return self.tabBar().tabText(self.currentIndex())



icons = {}

icons['arrow-down'] = '''<svg width="24" height="24" viewBox="0 0 24 24" fill="none" stroke="black" stroke-width="2" stroke-linecap="round"
    stroke-linejoin="round">
    <path d="m19 12 -7 7 -7 -7 h5 v-8 h4 v8 h5"/>
</svg>'''

icons['arrow-up'] = '''<svg width="24" height="24" viewBox="0 0 24 24" fill="none" stroke="black" stroke-width="2" stroke-linecap="round"
    stroke-linejoin="round">
    <path d="m5 12 7-7 7 7 h-5 v8 h-4 v-8 h-5" />
</svg>'''

icons['bulk-update'] = '''<svg width="24" height="24" viewBox="0 0 24 24" fill="none" stroke="black" stroke-width="2" stroke-linecap="round"
    stroke-linejoin="round">
  <path d="M2 2h8" />
  <path d="M2 7h8" />
  <path d="M2 12h8" />
  <path d="M2 17h8" />
  <path d="M2 22h8" />    <path
        d="M21.5 12a1 1 0 0 0-3.004-3.004l-4.01 4.012a2 2 0 0 0-.506.854l-.837 2.87a.5.5 0 0 0 .62.62l2.87-.837a2 2 0 0 0 .854-.506z" />

</svg>'''

icons['chart'] = '''<svg width="24" height="24" viewBox="0 0 24 24" fill="none" stroke="black" stroke-width="2" stroke-linecap="round"
    stroke-linejoin="round">
    <path d="M3 3 v18 h18" />
    <rect x="7" y="8" width="4" height="9" rx="0" />
    <rect x="15" y="5" width="4" height="12" rx="0" />
</svg>
'''

icons['close'] = '''<svg width="24" height="24" viewBox="0 0 24 24" fill="none" stroke="black" stroke-width="3" stroke-linecap="round"
    stroke-linejoin="round">
    <path d="M5 5 L21 21 M 5 21 L21 5" rx="2" ry="2" fill="none" />
</svg>'''

icons['color'] = '''<svg width="24" height="24" viewBox="0 0 24 24" fill="none" stroke="black" stroke-width="2" stroke-linecap="round"
    stroke-linejoin="round">
    <path
        d="M12 22a1 1 0 0 1 0-20 10 9 0 0 1 10 9 5 5 0 0 1-5 5h-2.25a1.75 1.75 0 0 0-1.4 2.8l.3.4a1.75 1.75 0 0 1-1.4 2.8z" />
    <circle cx="13.5" cy="6.5" r=".5" fill="currentColor" />
    <circle cx="17.5" cy="10.5" r=".5" fill="currentColor" />
    <circle cx="6.5" cy="12.5" r=".5" fill="currentColor" />
    <circle cx="8.5" cy="7.5" r=".5" fill="currentColor" />
</svg>'''

icons['exit'] = '''<svg width="24" height="24" viewBox="0 0 24 24" fill="none" stroke="black" stroke-width="2" stroke-linecap="round"
    stroke-linejoin="round">
    <path stroke-width="3" d="m12.2 7.2 v.01" />
    <path d="m13 20 v-3 l -2 -3 l 1.5 -4.5 l2 2 l2 1" />
    <path d="m5 17 h3 l 2 -3 l 1.5 -4.5 h-1 l-2 1 l-1 2" />
    <path d="m5 14.5 v -10.5 h14 v16" />
    <path d="m5 19.5 v 0.5 " />
</svg>'''

icons['export'] = '''<svg width="24" height="24" viewBox="0 0 24 24" fill="none" stroke="black" stroke-width="2" stroke-linecap="round"
    stroke-linejoin="round">
    <path d="M12 17V3" />
    <path d="M21 15v4a2 2 0 0 1-2 2H5a2 2 0 0 1-2-2v-4" />
    <path d="m7 8 5 -5 5 5" />
</svg>'''

icons['eye-off'] = '''<svg width="24" height="24" viewBox="0 0 24 24" fill="none" stroke="black" stroke-width="2" stroke-linecap="round"
    stroke-linejoin="round">
  <path d="M10.733 5.076a10.744 10.744 0 0 1 11.205 6.575 1 1 0 0 1 0 .696 10.747 10.747 0 0 1-1.444 2.49" />
  <path d="M14.084 14.158a3 3 0 0 1-4.242-4.242" />
  <path d="M17.479 17.499a10.75 10.75 0 0 1-15.417-5.151 1 1 0 0 1 0-.696 10.75 10.75 0 0 1 4.446-5.143" />
  <path d="m2 2 20 20" />
</svg>
'''

icons['eye'] = '''<svg width="24" height="24" viewBox="0 0 24 24" fill="none" stroke="black" stroke-width="2" stroke-linecap="round"
    stroke-linejoin="round">
  <path d="M2.062 12.348a1 1 0 0 1 0-.696 10.75 10.75 0 0 1 19.876 0 1 1 0 0 1 0 .696 10.75 10.75 0 0 1-19.876 0" />
  <circle cx="12" cy="12" r="3" />
</svg>
'''

icons['go-bottom'] = '''<svg width="24" height="24" viewBox="0 0 24 24" fill="none" stroke="black" stroke-width="2" stroke-linecap="round"
    stroke-linejoin="round">
    <path d="M5 20h14" />
    <path d="m18 12-6 6 -6 -6" />
    <path d="M12 4v13" />
</svg>
'''

icons['go-down'] = '''<svg width="24" height="24" viewBox="0 0 24 24" fill="none" stroke="black" stroke-width="2" stroke-linecap="round"
    stroke-linejoin="round">
    <path d="m19 12 -7 7 -7 -7" />
    <path d="M12 5v14" />
</svg>'''

icons['go-top'] = '''<svg width="24" height="24" viewBox="0 0 24 24" fill="none" stroke="black" stroke-width="2" stroke-linecap="round"
    stroke-linejoin="round">
    <path d="M5 4h14" />
    <path d="m18 12-6-6-6 6" />
    <path d="M12 7v13" />
</svg>
'''

icons['go-up'] = '''<svg width="24" height="24" viewBox="0 0 24 24" fill="none" stroke="black" stroke-width="2" stroke-linecap="round"
    stroke-linejoin="round">
    <path d="m5 12 7-7 7 7" />
    <path d="M12 19V5" />
</svg>'''

icons['help'] = '''<svg width="24" height="24" viewBox="0 0 24 24" fill="none" stroke="black" stroke-width="2" stroke-linecap="round"
    stroke-linejoin="round">
    <circle cx="12" cy="12" r="10" />
    <path d="M9.09 9a3 3 0 0 1 5.83 1c0 2-3 3-3 3 v2" />
    <line x1="12" x2="12.01" y1="18" y2="18" />
</svg>'''

icons['import'] = '''<svg width="24" height="24" viewBox="0 0 24 24" fill="none" stroke="black" stroke-width="2" stroke-linecap="round"
    stroke-linejoin="round">
    <path d="M12 15V3" />
    <path d="M21 15v4a2 2 0 0 1-2 2H5a2 2 0 0 1-2-2v-4" />
    <path d="m7 12 5 5 5-5" />
</svg>'''

icons['info'] = '''<svg width="24" height="24" viewBox="0 0 24 24" fill="none" stroke="black" stroke-width="2" stroke-linecap="round"
    stroke-linejoin="round">
    <path d="M12 17v-6" />
    <path d="M12 8h.01" />
    <circle cx="12" cy="12" r="10" />
</svg>'''

icons['list-chevrons-down-up'] = '''<svg width="24" height="24" viewBox="0 0 24 24" fill="none" stroke="black" stroke-width="2" stroke-linecap="round"
    stroke-linejoin="round">
  <path d="M3 2h8" />
  <path d="M3 7h8" />
  <path d="M3 12h8" />
  <path d="M3 17h8" />
  <path d="M3 22h8" />
  <path d="m15 5 3 3 3-3" />
  <path d="m15 19 3-3 3 3" />
</svg>'''

icons['list-chevrons-up-down'] = '''<svg width="24" height="24" viewBox="0 0 24 24" fill="none" stroke="black" stroke-width="2" stroke-linecap="round"
    stroke-linejoin="round">
  <path d="M3 2h8" />
  <path d="M3 7h8" />
  <path d="M3 12h8" />
  <path d="M3 17h8" />
  <path d="M3 22h8" />
  <path d="m15 8 3-3 3 3" />
  <path d="m15 16 3 3 3-3" />
</svg>
'''

icons['list-minus'] = '''<svg width="24" height="24" viewBox="0 0 24 24" fill="none" stroke="black" stroke-width="2" stroke-linecap="round"
    stroke-linejoin="round">
  <path d="M3 2h8" />
  <path d="M3 7h8" />
  <path d="M3 12h8" />
  <path d="M3 17h8" />
  <path d="M3 22h8" />
  <path d="M21 12h-6" />
</svg>
'''

icons['list-plus'] = '''<svg width="24" height="24" viewBox="0 0 24 24" fill="none" stroke="black" stroke-width="2" stroke-linecap="round"
    stroke-linejoin="round">
  <path d="M3 2h8" />
  <path d="M3 7h8" />
  <path d="M3 12h8" />
  <path d="M3 17h8" />
  <path d="M3 22h8" />
  <path d="M18 9v6" />
  <path d="M21 12h-6" />
</svg>
'''

icons['lock'] = '''<svg width="24" height="24" viewBox="0 0 24 24" fill="none" stroke="black" stroke-width="2" stroke-linecap="round"
    stroke-linejoin="round">
    <circle cx="12" cy="16" r="2" />
    <rect width="12" height="12" x="6" y="10" rx="2" />
    <path d="M8 10V8 a4 5 0 0 1 8 0 v1" />
</svg>'''

icons['menu'] = '''<svg width="24" height="24" viewBox="0 0 24 24" fill="none" stroke="black" stroke-width="2" stroke-linecap="round"
    stroke-linejoin="round">
    <path d="m4 7 h16"/>
    <path d="m4 12 h16"/>
    <path d="m4 17 h16"/>
</svg>'''

icons['moon'] = '''<svg width="24" height="24" viewBox="0 0 24 24" fill="none" stroke="black" stroke-width="2" stroke-linecap="round"
    stroke-linejoin="round">
    <path
        d="M20.985 12.486a9 9 0 1 1-9.473-9.472c.405-.022.617.46.402.803a6 6 0 0 0 8.268 8.268c.344-.215.825-.004.803.401" />
</svg>'''

icons['new'] = '''<svg width="24" height="24" viewBox="0 0 24 24" fill="none" stroke="black" stroke-width="2" stroke-linecap="round"
    stroke-linejoin="round">
    <path stroke="red" stroke-width="1" d="m 0 0 h24 v24 h-24 v-24" />
</svg>'''

icons['paste-csv'] = '''<svg width="24" height="24" viewBox="0 0 24 24" fill="none" stroke="black" stroke-width="2" stroke-linecap="round"
    stroke-linejoin="round">
    <path stroke-width="2" d="m12 22 h-8v-20h12l4 4 v6" />
    <path stroke-width="2" d="m7 5 l 5 9" />
    <path stroke-width="2" d="m12 5 l-5 9" />
    <path stroke-width="2" d="m14 19 l3 3 l3 -3" />
    <path stroke-width="4" d="m17 14 v5" />
</svg>'''

icons['print'] = '''<svg width="24" height="24" viewBox="0 0 24 24" fill="none" stroke="black" stroke-width="2" stroke-linecap="round"
    stroke-linejoin="round">
    <path d="M6 18H4a2 2 0 0 1-2-2v-5a2 2 0 0 1 2-2h16a2 2 0 0 1 2 2v5a2 2 0 0 1-2 2h-2" />
    <rect x="6" y="14" width="12" height="8" rx="1" />
    <rect x="6" y="2" width="12" height="7" rx="1"/>

</svg>'''

icons['q2gui'] = '''<svg width="24" height="24" viewBox="0 0 24 24" fill="none" stroke="black" stroke-width="2" stroke-linecap="round"
    >
    <rect stroke="white" fill="white" stroke-width="1" x="1" y="1" width="22" height="22" rx="100" ry="100" />
    <rect x="11" y="13" width="5.5" height="5.5" transform="rotate(-135, 10, 12)" fill="black"/>
    <rect x="11" y="13" width="5.5" height="5.5" transform="rotate(45, 10, 12)" fill="black"/>
    <rect x="11.4" y="14" width="5.5" height="5.5" transform="rotate(-45, 9.4, 12)" fill="black"/>
</svg>'''

icons['refresh'] = '''<svg width="24" height="24" viewBox="0 0 24 24" fill="none" stroke="black" stroke-width="2" stroke-linecap="round"
    stroke-linejoin="round">
    <path d="M21 12a9 9 0 1 1-9-9c2.52 0 4.93 1 6.74 2.74L21 8" />
    <path d="M21 3v5h-5" />
</svg>
'''

icons['restore'] = '''<svg width="24" height="24" viewBox="0 0 24 24" fill="none" stroke="black" stroke-width="2" stroke-linecap="round"
    stroke-linejoin="round">
    <rect x="4" y="11" width="11" height="11" rx="0" ry="0" />
    <path d="M8 10 V6 H20 V18 H16" rx="2" ry="2" fill="none" />
</svg>'''

icons['row-copy'] = '''<svg width="24" height="24" viewBox="0 0 24 24" fill="none" stroke="black" stroke-width="2" stroke-linecap="round"
    stroke-linejoin="round">
    <path stroke-width="2" d="m21 18v1h-15v-17h11l4 4v12" />
    <path stroke-width="2" d="m19 22v0h-16v-18" />
</svg>'''

icons['row-edit'] = '''<svg width="24" height="24" viewBox="0 0 24 24" fill="none" stroke="black" stroke-width="2" stroke-linecap="round"
    stroke-linejoin="round">
    <path stroke-width="2" d="m20 18v4h-16v-20h12l4 4" />
    <path
        d="M20 12a1 1 0 0 0-3.004-3.004l-4.01 4.012a2 2 0 0 0-.506.854l-.837 2.87a.5.5 0 0 0 .62.62l2.87-.837a2 2 0 0 0 .854-.506z" />
    <path d="M7 19h2" />
</svg>'''

icons['row-new'] = '''<svg width="24" height="24" viewBox="0 0 24 24" fill="none" stroke="black" stroke-width="2" stroke-linecap="round"
    stroke-linejoin="round">
    <path stroke-width="2" d="m20 18v4h-16v-20h12l4 4v12" />
</svg>'''

icons['row-remove'] = '''<svg width="24" height="24" viewBox="0 0 24 24" fill="none" stroke="black" stroke-width="2" stroke-linecap="round"
    stroke-linejoin="round">
    <path stroke-width="2" d="m12 22 h-8v-20h12l4 4 v6" />
    <path d="M14 14 l7 7" />
    <path d="M21 14 l-7 7" />
</svg>'''

icons['row-view'] = '''<svg width="24" height="24" viewBox="0 0 24 24" fill="none" stroke="black" stroke-width="2" stroke-linecap="round"
    stroke-linejoin="round">
    <path d="M2.062 12.348a1 1 0 0 1 0-.696 10.75 10.75 0 0 1 19.876 0 1 1 0 0 1 0 .696 10.75 10.75 0 0 1-19.876 0" />
    <circle cx="12" cy="12" r="3" />
</svg>'''

icons['select'] = '''<svg width="24" height="24" viewBox="0 0 24 24" fill="none" stroke="black" stroke-width="2" stroke-linecap="round"
    stroke-linejoin="round">
    <path stroke-width="2" d="m4 13 l 6 6 l11 -11" />
</svg>'''

icons['star'] = '''<svg width="24" height="24" viewBox="0 0 24 24" fill="none" stroke="black" stroke-width="2" stroke-linecap="round"
    stroke-linejoin="round">
    <path d="M12 2 L14.85 8.63 L22 9.27 L16.5 14.14 L17.6 21 L12 17.77 L6.4 21 L7.5 14.14 L2 9.27 L9.15 8.63 Z" />
</svg>
'''

icons['sun'] = '''<svg width="24" height="24" viewBox="0 0 24 24" fill="none" stroke="black" stroke-width="2" stroke-linecap="round"
    stroke-linejoin="round">
    <circle cx="12" cy="12" r="4" />
    <path d="M12 2v2" />
    <path d="M12 20v2" />
    <path d="m4.93 4.93 1.41 1.41" />
    <path d="m17.66 17.66 1.41 1.41" />
    <path d="M2 12h2" />
    <path d="M20 12h2" />
    <path d="m6.34 17.66-1.41 1.41" />
    <path d="m19.07 4.93-1.41 1.41" />
</svg>    
'''

icons['time'] = '''<svg width="24" height="24" viewBox="0 0 24 24" fill="none" stroke="black" stroke-width="2" stroke-linecap="round"
    stroke-linejoin="round">
    <path d="M12 6v6l4-2" />
    <circle cx="12" cy="12" r="10" />
</svg>
'''

icons['tools'] = '''<svg width="24" height="24" viewBox="0 0 24 24" fill="none" stroke="black" stroke-width="2" stroke-linecap="round"
    stroke-linejoin="round">
    <path d="M14.7 6.3a1 1 0 0 0 0 1.4l1.6 1.6a1 1 0 0 0 1.4 0l3.106-3.105c.32-.322.863-.22.983.218a6 6 0 0 1-8.259 7.057l-7.91 7.91a1 1 0 0 1-2.999-3l7.91-7.91a6 6 0 0 1 7.057-8.259c.438.12.54.662.219.984z"/>
</svg>
'''

icons['unlock'] = '''<svg width="24" height="24" viewBox="0 0 24 24" fill="none" stroke="black" stroke-width="2" stroke-linecap="round"
    stroke-linejoin="round">
    <circle cx="9" cy="16" r="2" />
    <rect width="12" height="12" x="3" y="10" rx="2" />
    <path d="M12 10V7a5 5 0 0 1 9.33-2.5" />
</svg>'''
